//
// Windows Socket Client to fetch files from websites
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "sockutil.h"
#include "parsecmd.h"

#define LF 10
#define CR 13

#define BUFFER_SIZE 16384

//
// wget - the main routine
//
BOOL wget (char * szUrl, char * szFileName, BOOL bHeaderRecs)
{
    // URL data
    char szProtocol [MAX_PATH] = "tcp";
    char szServName [MAX_PATH] = "http";
    char szHostName [MAX_PATH] = "localhost";
    int iPortNumber = 80;
    char szPathName [MAX_PATH] = "/";

    char szBuffer [BUFFER_SIZE];

    SOCKET socketHandle;
    int socketResult;

    int fileHandle;
    int fileResult;
    
    // initialize WinSock library
    WinSockInitialize();

    // Create a stream socket
    socketHandle = CreateSocket (SOCK_STREAM);
    if (socketHandle == INVALID_SOCKET)
    {
        WSACleanup();
        return(1);
    }

    socketResult = BindSocketToAddr (DEFAULT_PORT, socketHandle, NULL, NULL, 0, TRUE);
    if (socketResult)
    {
        closesocket(socketHandle);
        WSACleanup();
        return(1);
    }

    // Parse URL
    ParseHttpUrl ((char *) szUrl, (char *) szFileName,
                  (char *) szHostName, (char *) szPathName, &iPortNumber);

    // Connect to the Server
#ifdef _DEBUG
    printf ("Connecting to Server: %s on Host: %s Protocol: %s\n", 
            szServName, szHostName, szProtocol);
#endif

    if ((iPortNumber == 0) || (iPortNumber == 80))
        socketResult = ConnectToServer (REMOTE_ADDR, socketHandle, szHostName, szServName, szProtocol, 0);
    else
        socketResult = ConnectToServer (REMOTE_PORT, socketHandle, szHostName, NULL, szProtocol, iPortNumber);

    if (socketResult != 0)
    {
        closesocket(socketHandle);
        WSACleanup();
        return(1);
    }

    // get file from http server
    if (bHeaderRecs)
        sprintf (szBuffer, "GET %s HTTP/1.0%c%c", szPathName, LF, LF);
    else
        sprintf (szBuffer, "GET %s%c%c", szPathName, LF, LF);
    socketResult = send (socketHandle, szBuffer, strlen(szBuffer), 0);
    if (socketResult == SOCKET_ERROR)
        printf ("Error: %d in send\n", WSAGetLastError());
    else
    {
        // open local file
        if ((fileHandle = _open (szFileName, _O_CREAT | _O_WRONLY | 
                                             _O_TRUNC | _O_BINARY)) !=  -1)
        {
            // read (first part of) server's reply
            socketResult = recv (socketHandle, szBuffer, sizeof(szBuffer), 0);
            if (socketResult == SOCKET_ERROR)
                printf ("Error: %d in recv\n", WSAGetLastError());

            while (socketResult > 0)
            {
                // write to local file
                if ((fileResult = _write (fileHandle, szBuffer, socketResult)) == -1)
                {
                    printf ("Error: in write to file\n");
                    break;
                }

                // read next block (if available) from web-page
                socketResult = recv (socketHandle, szBuffer, sizeof(szBuffer), 0);
                if (socketResult == SOCKET_ERROR)
                {
                    printf ("Error: %d in recv\n", WSAGetLastError());
                    break;
                }
            }

            // close local file
            _close (fileHandle);
        }
    }

    // Close the socket
    closesocket (socketHandle);

    // Cleanup and return
    WSACleanup();

    return TRUE;
}

//
// MAIN - the program
//
int main (int argc, char **argv)
{
    // URL data
    char szUrl [MAX_PATH] = "";
    char szFileName [MAX_PATH] = "";
    BOOL bHeaderRecs = FALSE;

    // Parse command line
    ParseCmdLine (argc, argv, (char *) szUrl, (char *) szFileName, &bHeaderRecs);

    if (!wget(szUrl, szFileName, bHeaderRecs))
    {
        return 1;
    }

    return 0;
}

